clear
clc
close all

addpath('functions'); % set path to helper functions

load('result_1e4.mat')

fevd_news = mean(squeeze(Draws_FEVDs(:,5,:,:)),3)';
fevd_tfp = mean(squeeze(Draws_FEVDs(:,6,:,:)),3)';

Draws_IRFs_cum   = nan(ntilde,ntilde,Hz,numDesireddraws);
Draws_FEVDs_cum  = nan(ntilde,ntilde,Hz,numDesireddraws);

for i = 1:numDesireddraws
    
    IRFs =  squeeze(Draws_IRFs(:,:,:,i));
    Draws_IRFs_cum(:,:,:,i)  = cumsum(IRFs,3);
    Draws_FEVDs_cum(:,:,:,i) = varianceDecompositionOfVAR(Draws_IRFs_cum(:,:,:,i),Hz-1);
           
end

targetshock = [n+1;n+2];
LineWidthp  = 3;
bands68 = [16,84];
bands90 = [5,95];
plotx   = 0:1:Hz-1;
for j = 1:length(targetshock)
    targetshockj = targetshock(j);
    figure
    for i = 1:ntilde
        subplot(ceil(ntilde/3),3,i)
        relevantIRFs = squeeze(Draws_IRFs_cum(i,targetshockj,:,:));
        medianIRF    = prctile(relevantIRFs,50,2);
        IRFband68    = prctile(relevantIRFs,bands68,2);
        IRFband90    = prctile(relevantIRFs,bands90,2);
        plot(plotx,medianIRF,'LineWidth',LineWidthp,'Color','b')
        hold on
        [~,~]=jbfill(plotx,IRFband68(:,1)',IRFband68(:,2)',rgb('mediumblue'),rgb('mediumblue'),0,0.3);
        hold on
        [~,~]=jbfill(plotx,IRFband90(:,1)',IRFband90(:,2)',rgb('mediumblue'),rgb('mediumblue'),0,0.1);
        hold on
        plot(plotx,zeros(Hz,1),'LineWidth',1,'Color',[0.5 0.5 0.5]);
        axis tight;
        if i <= n
            title(varnames{i})
        end
    end
end

for j = 1:length(targetshock)
    targetshockj = targetshock(j);
    figure
    for i = 1:ntilde
        subplot(ceil(ntilde/3),3,i)
        relevantFEVDs = squeeze(Draws_FEVDs_cum(i,targetshockj,:,:));
        medianFEVD    = prctile(relevantFEVDs,50,2);
        meanFEVD      = mean(relevantFEVDs,2);
        FEVDband68    = prctile(relevantFEVDs,bands68,2);
        FEVDband90    = prctile(relevantFEVDs,bands90,2);
        plot(plotx,meanFEVD,'LineWidth',LineWidthp,'Color','b')
        hold on
        [~,~]=jbfill(plotx,FEVDband68(:,1)',FEVDband68(:,2)',rgb('mediumblue'),rgb('mediumblue'),0,0.3);
        hold on
        [~,~]=jbfill(plotx,FEVDband90(:,1)',FEVDband90(:,2)',rgb('mediumblue'),rgb('mediumblue'),0,0.1);
        hold on
        plot(plotx,zeros(Hz,1),'LineWidth',1,'Color',[0.5 0.5 0.5]);
        axis tight;
        if i <= n
            title(varnames{i})
        end
    end
end

fevd_news_cum = mean(squeeze(Draws_FEVDs_cum(:,5,:,:)),3)';
fevd_tfp_cum = mean(squeeze(Draws_FEVDs_cum(:,6,:,:)),3)';
fevd_m1_cum = mean(squeeze(Draws_FEVDs_cum(:,7,:,:)),3)';
fevd_m2_cum = mean(squeeze(Draws_FEVDs_cum(:,8,:,:)),3)';
fevd_m_cum  = fevd_m1_cum + fevd_m2_cum;

figure
plot(fevd_news_cum(:,n+1),'LineWidth',LineWidthp,'Color','b')
hold on
plot(fevd_tfp_cum(:,n+1),'LineWidth',LineWidthp,'Color','k')
hold on
plot(fevd_m_cum(:,n+1),'LineWidth',LineWidthp,'Color','r')
legend('news shock','tfp shock','measurement error')
title('Patent IV Decompostion')

figure
plot(fevd_news_cum(:,n+2),'LineWidth',LineWidthp,'Color','b')
hold on
plot(fevd_tfp_cum(:,n+2),'LineWidth',LineWidthp,'Color','k')
hold on
plot(fevd_m_cum(:,n+2),'LineWidth',LineWidthp,'Color','r')
legend('news shock','tfp shock','measurement error')
title('TFP IV Decompostion')
