clearvars
close all
clc

rng('default')
rng(0);         % set seed

addpath('functions')

rawdata = xlsread('data.xlsx','main','B2:BC303');
rawwage = xlsread('data.xlsx','wage','B2:AD303');
rawwage = rawwage(:,2:end);

dates = zeros(304,1);
count = 0;
for i = 1947:2022
    for j = 1:4
        count = count +1;
        dates(count,1) = datenum(i,j*3,1);
    end
end
dates = dates(1:end-2);
% datevec(dates)

dateselect = 1;
switch dateselect
    case 1
        start_date = datenum(1960,3,1);
        end_date   = datenum(2018,6,1);
end

start      = find(dates==start_date);
finish     = find(dates==end_date);

dates      = dates(start:finish,:);
% datevec(dates)

DATASET.TSERIES  = rawdata(start:finish,:);
DATASET.LABEL    = {'dtfp__util','logtfp__util','logPatent','Patent__diff','FFR', ...
                    'GS5','GS10','GDPDEF','News_Instrument1','Nasdaq',   ...    % 10
                    'gz__spread','ebp','default__risk','SP500','GDP', ...
                    'share__c1','share__c2','share__i1','share__i2','Consumption', ...  % 20
                    'Investment','Total hours','NFB hours','Productivity','Amazon__log', ...
                    'Amazon__diff','PRS85006023','CE16OV','CNP16OV','Hours', ...      % 30
                    'GDPDEFg','spread','CPI','realSP500','realSP500p', ...
                    'TFP__AEJM','GDP__AEJM','Investment__AEJM','Consumption__AEJM','Hours__AEJM', ... % 40
                    'SP500__AEJM','Inflation__AEJM','ebp__AEJM','gz__spr__AEJM','defaultrisk__AEJM', ...
                    'tfp__QE','gdp__QE','con__QE','inv__QE','emp__QE', ...                                % 50
                    'NYSE__QE','tfpu__QE','Productivity__QE','dtfp__QE'};
DATASET.MAP      = containers.Map(DATASET.LABEL,1:size(DATASET.TSERIES,2));

var_select_vars  = {'Productivity__QE','gdp__QE','con__QE','inv__QE','emp__QE','NYSE__QE'};
VARData0         = DATASET.TSERIES(:,cell2mat(values(DATASET.MAP,var_select_vars)));
change.log       = [0 0 0 0 0 0];
VARData          = VARData0;
for i = 1:length(var_select_vars)
    if change.log(i) == 1
        VARData(:,i) = log(VARData0(:,i))*100;
    end
end

%% max share setting
%ID of shock 1
targ_var1 = [1];                    %Variable to target in yxdat
kmin1     = 20;                     %Starting period to include in maximand
kmax1     = 80;                     %Last period to include in maximization
norm_idx1 =  1 ;                    %Sign normalized for what variable's response
hor_idx1  =  25;                    %Horizon of normalization

%ID of shock 2
targ_var2 = [1];                                                         
kmin2     = 0;
kmax2     = 0;
norm_idx2 = 1;   
hor_idx2  = 0;  

%% SVAR setting
opt.p          = 4;    % No. of lags in VAR
opt.const      = 1;    % const = 1 if constant in VAR, = 0 otherwise
opt.Kdraws     = 1000;  % No. of draws from posterior inference
opt.H          = 40;   % Terminal horizon for IR

% Data for VAR for y_t.
YY = VARData(opt.p+1:end,:); % y_t 
XX = lagmatrix(VARData,1:opt.p); % Matrix of regressors in VAR for y_t
XX = XX(opt.p+1:end,:); % Drop initial missing observations
if opt.const == 1 % Add constant to matrix of regressors
    XX = [XX ones(length(XX),1)];
end

%% Conduct posterior inference.
n     = size(YY,2); % Number of variables
m     = size(XX,2); % Number of parameters in each equation for y_t
T     = length(YY); % Number of observations used in estimating VAR

phiHat_B     = (XX'*XX)\XX'*YY;
phiHat_Bv    = phiHat_B(:);
phiHat_S     = (YY - XX*phiHat_B)'*(YY-XX*phiHat_B);
phiHat_Sigma = (1/T)*phiHat_S; % MLE of innovation covariance matrix
phiHat_P     = (XX'*XX)\eye(m);
phiHat_cholP = chol(phiHat_P,'lower');
PphiTilde    = YY'*YY - phiHat_B'*(XX'*XX)*phiHat_B;
PphiTilde    = (PphiTilde'+PphiTilde)*0.5;

% Save Results
B = nan(m*n,opt.Kdraws);
SigmatrQ = nan(n,n,opt.Kdraws);

for i = 1:opt.Kdraws
    
    % Draw reduced-form parameters for VAR for y_t.
    phi_Sigma      = iwishrnd(PphiTilde,T);
    phi_Sigmatr    = chol(phi_Sigma,'lower');
    phi_Sigmatrinv = phi_Sigmatr\eye(n);
    phi_B          = phiHat_Bv + kron(phi_Sigmatr,phiHat_cholP)*randn(m*n,1);
    
    beta_draws = reshape(phi_B,m,n);
    bB         = beta_draws(1:end-1,:);
    
    Pnew = identify_shocks(companion_form(bB'),phi_Sigmatr,targ_var1,targ_var2,kmin1,kmin2,kmax1,kmax2,norm_idx1,norm_idx2,hor_idx1,hor_idx2);
    % Pnew*Pnew' - phi_Sigma 
    
    % Save Results
    B(:,i) = phi_B;
    SigmatrQ(:,:,i) = Pnew;
    
end

%% Plot IRF
shockname = cell(5,1);
shockname{1} = 'news';
shockname{2} = 'tfp';

hmax        = opt.H;
bands       = [16,50,84];   
medianwidth = 3;
x           = 0:hmax;
varnames    = var_select_vars;

shockind = [1; 2];
plot_irf;

%% Plot FEVD
plot_fevd;
plot_fevd_diff

%% plot shock quantile
plot_shock_quantile

%% plot correlation
plot_correlation    