function [nu_bar, Phi_bar_inv, mu_bar, Omega_bar_inv] = SimsZhaPriorParameters(Y, X, n_lags, periods_per_year, hyperparameters, variance_scale)
%
% Y - endogenous data, each row is an observation
% X - predetermined data, each row is an observation
% n_lags - number of lags
% periods_per_year - number periods per year (monthly = 12, quarterly = 4) 
% hyperparameters - parameters controlling prior
% variance_scale - optional scaling parameter
%
% nu_bar - generalized gaussian parameter
% Phi_bar_inv - inverse of generalized gaussian parameter
% mu_bar - matrix gaussian parameter
% Omega_bar_inv - inverse of matrix gaussian parameter
%

if nargin == 6
    [Y_bar, X_bar] = SimsZhaDummyObservations(Y, X, n_lags, periods_per_year, hyperparameters, variance_scale);
else
    [Y_bar, X_bar] = SimsZhaDummyObservations(Y, X, n_lags, periods_per_year, hyperparameters);
end

nu_bar=size(Y,2);

Omega_bar_inv=X'*X;

mu_bar=Omega_bar_inv\(X_bar'*Y_bar);

Phi_bar_inv=Y_bar'*Y_bar - mu_bar'*(Omega_bar_inv*mu_bar); 
