clear
clc
close all

addpath('functions') % set path to helper functions
addpath('matlab_720_v3')

tstart = datetime('now');  % start time

%%
load('resultSS0.mat')

rank_period = 1;

conjugateprior  = 'none';        % when equal to 'none' the volume element is not computed
                                       % when equal to 'structural' the volume element is computed
printi = 100;
    
for sigmai = 1:sigman
    for thetai = 1:thetan
        
        theta1 = thetavalue(thetai,1);
        theta2 = thetavalue(thetai,2);
        B1matrix = [ 1  theta1 ;
                     theta2  1 ];
        vsigma1 = sigmavalue(sigmai,1);
        vsigma2 = sigmavalue(sigmai,2);
        vsigma  = [ vsigma1  0;
                    0        vsigma2 ]; 
        
        % VAR Specification
        VAR_Specification

        % Proxy restrictions
        % The zero restrictions will be multiplied by a function
        % F(\tilde{A}_{0},\tilde{A}_{+}) = [\tilde{A}_{0};
        %                                  (\tilde{A}_{0})^{-1\prime}].
        get_proxy_restrictions;
        Z{1} = Z{2};
        
        % Get all zero restrictions under the structural parameterization. That is block restrictions, proxy restrictions, and additional restrictions if any.
        % In this step we collect all the zero restrictions in an array called Zstructural. The zero restrictions will be multiplied by a function
        % F(\tilde{A}_{0},\tilde{A}_{+}) = [\tilde{A}_{0};
        %                                  (\tilde{A}_{0})^{-1\prime};
        %                                   \tilde{A}_{+}].
        get_all_zero_restrictions;
        Zstructural{1} = Zstructural{2};
        
        ind1 = (sigmai-1)*thetan+thetai;
        for mci = 1:mcnumber
            
            disp(mci)
            % simulate shock
            u_simulate = squeeze(trueshocksave(ind1,mci,:,1:n));
            v_simulate = squeeze(trueshocksave(ind1,mci,:,n+1:n+k));
           
            % simulate data
            vardata    = u_simulate*Bmatrix';
            % simulate proxy
            proxies    = u_simulate(:,n-k+1:n)*B1matrix' + v_simulate*vsigma';
            
            % Estimate
            mainestimate_SR_NoIV
            
            a0save(ind1,mci,:,:,:) = A0tilde;
            Aplussave(ind1,mci,:,:,:) = Aplustilde;
            countsave(ind1,mci) = count;
            Thetasave(ind1,mci,:) = Theta;
            rhosave(ind1,mci,:,:) = corrs;
            
        end
    end 
end

result_table = nan(n,6);
for sigmai = 1:sigman
    for thetai = 1:thetan
        ind1 = (sigmai-1)*thetan+thetai;
        for j = 1:n
            result_table(j,ind1) = median(mean(rhosave(ind1,:,:,j),2));
        end
    end
end

numString = [];
for ii = 1:11
    numString = [numString '%8.2f & '];
end
numString = [numString '%8.2f'];

FID = fopen(strcat(['correlation_NSR',num2str(rank_period),'.tex']),'w');
fprintf(FID,strcat('%s',numString,'  \\\\  \n'),'$NSR_1$ & ',[mean(result_table(2,:))*ones(1,6),mean(result_table(3,:))*ones(1,6)]);

%%
save resultNSR1 trueshocksave a0save Aplussave countsave Thetasave rhosave T n k mcnumber numDesireddraws sigmavalue thetavalue sigman thetan Bmatrix rank_period

%%
tend = datetime('now');    % end time
computingtime = tend-tstart; % computing time
disp(computingtime)