numdata = [vardata,proxies];
Ns      = zeros(size(numdata));
id1     = randperm(T);
id2     = randperm(T);

for si = 1:kn1_number
    if u_simulate(id1(si),n-1) > 0
        Ns(id1(si),n-1) = 1;
    elseif u_simulate(id1(si),n-1) < 0
        Ns(id1(si),n-1) = -1;
    end
end
    
for si = 1:kn2_number
    if u_simulate(id2(si),n) > 0
        Ns(id2(si),n) = 1;
    elseif u_simulate(id2(si),n) < 0
        Ns(id2(si),n) = -1;
    end
end

%% Proxy-SVAR Specification
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Write data in model's notation
% \tilde{y}_{t} \tilde{A}0 = \tilde{x}_{t} \tilde{Aplus} + \tilde{constant} + \tilde{e}_{t} for t=1...,T;
% tilde{x}_(t)             = [\tilde{y}_{t-1}, ... , \tilde{y}_{t-p}, constant];
% \tilde_{y_{0:-p+1}       = numdata(1:n_lags,:);
yt  = numdata(n_lags+1:end,:);      % define y transpose as in Rubio-Ramirez, Waggoner, and Zha(2011)
T   = size(yt,1);                   % sample size
xt  = zeros(T,ntilde*n_lags+nex);
for i = 1:n_lags
    xt(:,ntilde*(i-1)+1:ntilde*i) = numdata((n_lags-(i-1)):end-i,:) ;
end
if nex >= 1
    xt(:,ntilde*n_lags+nex)=ones(T,1);
end
Y  = yt;
X  = xt;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Main engine
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute the posterior parameters of the Normal-Generalized-Normal distribution over the structural parameters
SBVAR = SetupSBVAR_Proposal(n_lags, nu, Phi, Psi, Omega_inv, nu_bar, Phi_bar, Psi_bar, Omega_bar_inv, Y, X, Uj, Vj);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set up preliminaries to draw from the triangular-block parameterization and to compute the volume elements

% get the dimensions of (omega_1 and omega_2 as well as W_1 and W_2)
get_dim_Ws_and_info;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Declare objects that will store useful results
A0tilde_save          = nan(ntilde,ntilde,numDesireddraws);   % matrix to store A0tilde
Aplustilde_save       = nan(mtilde,ntilde,numDesireddraws);   % matrix to store Aplustilde
uw_save               = nan(1,numDesireddraws);               % unnormalized importance sampler weights
theta_save            = nan(1,numDesireddraws);

% declare functions to compute the volume elements in both directions
fs       = @(x)ff_h(x,info);
rs       = @(x)ZeroRestrictions(x,info);
fs_inv   = @(x)ff_h_inv(x,info);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize iterations for draws

record = 0;
count  = 0;

while record < numDesireddraws
    
    count  = count + 1;
    if mod(count,printi) == 0
        disp(strcat(['count = ',num2str(count)]))
        disp(strcat(['record = ',num2str(record)]))
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Draw \tilde{Llambda}_{0}, \tilde{Llambda}_{+}
    [Llambda0tilde, Llambdaplustilde] = DrawPosterior_SBVAR(SBVAR);
    
    Calculate_M;
    
    SigmatrM = inv(Llambda0tilde)'*M;
    invSigmatrM = inv(SigmatrM);
    
    vardata2 = numdata - kron(ones(T,1),Llambdaplustilde/(Llambda0tilde));
    h1 = invSigmatrM(n-1:n,:)*vardata2(id1(1:kn1_number),:)';
    h11 = -h1(1,:)./h1(2,:);
    sign1 = Ns(id1(1:kn1_number),n-1);
    
    btheta = [];
    stheta = [];
    for i = 1:kn1_number
        if sign1(i)*h1(2,i) > 0
            stheta = [stheta;h11(i)];
        else
            btheta = [btheta;h11(i)];
        end
    end
            
    h1 = invSigmatrM(n-1:n,:)*vardata2(id2(1:kn1_number),:)';
    h11 = h1(2,:)./h1(1,:);
    sign1 = Ns(id2(1:kn1_number),n);

    for i = 1:kn1_number
        if sign1(i)*h1(1,i) < 0
            stheta = [stheta;h11(i)];
        else
            btheta = [btheta;h11(i)];
        end
    end
     
    if isempty(stheta)
        mintheta = - pi/2;
    else
        mintheta = atan(max(stheta));
    end
    if isempty(btheta)
        maxtheta = pi/2;
    else
        maxtheta = atan(min(btheta));
    end
    
    if maxtheta > mintheta
        
        record = record + 1;
                
        theta  = mintheta + (maxtheta-mintheta)*rand(1);
        Qstar  = [ 1 0 0 ;
                   0 cos(theta) -sin(theta);
                   0 sin(theta)  cos(theta) ];
        Q      = M(1:n,1:n)*Qstar;
        
        structpara = f_h([reshape(Llambda0tilde,ntilde*ntilde,1);reshape(Llambdaplustilde,mtilde*ntilde,1);...
                          vec(Q); vec(eye(k))],info);

        A0tilde_save(:,:,record)    = reshape(structpara(1:ntilde*ntilde),ntilde,ntilde);
        Aplustilde_save(:,:,record) = reshape(structpara(1+ntilde*ntilde:end),mtilde,ntilde);
        theta_save(record) = theta;
        
        switch conjugateprior    
            case 'structural'
                storevefZ_inv     = LogVolumeElement(fs_inv,structpara,rs);
                log_target        = log_NGN(SBVAR.nu_tilde,SBVAR.Phi_tilde,SBVAR.Psi_tilde,SBVAR.Omega_tilde_inv,reshape(structpara(1:ntilde*ntilde),ntilde,ntilde),reshape(structpara(1+ntilde*ntilde:end),mtilde,ntilde));     
                log_prop          = log_NGN(SBVAR.nuhat,SBVAR.Phi_hat,SBVAR.Psi_hat,SBVAR.Omega_hat_inv,Llambda0tilde,Llambdaplustilde)  + storevefZ_inv;
                uw_save(:,record) = exp(log_target- log_prop);
            case 'none'
                uw_save(:,record) = 1;
        end
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% compute the normalized weights and estimate the effective sample size of the importance sampler
w_IP  = uw_save/sum(uw_save);
ne    = floor(1/sum(w_IP.^2));

% useful definitions to store relevant objects
A0tilde       = zeros(ntilde,ntilde,numDesireddraws);            % define array to store weighted \tilde{A}_0
Aplustilde    = zeros(ntilde*n_lags+nex,ntilde,numDesireddraws); % define array to store weighted \tilde{A}_plus
Theta         = nan(numDesireddraws,1);

% initialize counter to track the state of the importance sampler
corrs   = nan(numDesireddraws,n);
parfor s = 1:numDesireddraws
  
    is_draw             = randsample(1:size(w_IP,2),1,true,w_IP);
    A0tilde(:,:,s)      = A0tilde_save(:,:,is_draw);
    Aplustilde(:,:,s)   = Aplustilde_save(:,:,is_draw);
    Theta(s)            = theta_save(is_draw);
    Shocks_identify     = Y*A0tilde_save(:,:,is_draw) -X*Aplustilde_save(:,:,is_draw);
    for ci = 1:n
        temp = corrcoef([Shocks_identify(:,ci) u_simulate(:,ci)]);
        corrs(s,ci) = temp(2,1);
    end      
end