function x = UnivariateGeneralizedGaussian(nu, phi)
%
% The univariate generalized gaussian distribution, with parameters nu >= 1
% and phi > 0, has a density of the form:
%
%                 csnt*|x|^(nu-1)*exp(-0.5*x^2/phi)
%
% where csnt = (2*phi)^(-0.5*nu)*gamma(0.5*nu).
%

b=sqrt(2*phi);
c=sqrt(phi*(nu-1));
if nu > 1.0
    d=(nu-1)*(0.5-log(c));
else
    d=0.0;
end

t=tan(3.14159265358979323846*rand);       % 3.14159265358979323846 = pi
if rand < 0.5
    x=b*t+c;
    s=t+2.0*c/b;
else
    x=b*t-c;
    s=t-2.0*c/b;
end

while (x == 0.0) || (log(rand*(1.0/(1.0+t*t)+1.0/(1.0+s*s))) > d + (nu-1)*log(abs(x)) - 0.5*x*x/phi)
    t=tan(3.14159265358979323846*rand);
    if rand < 0.5
        x=b*t+c;
        s=t+2.0*c/b;
    else
        x=b*t-c;
        s=t-2.0*c/b;
    end
end
