clear
clc
close all

addpath('functions') % set path to helper functions
addpath('matlab_720_v3')

tstart = datetime('now');  % start time

%%
T = 100;   % sample size
n = 3;     % number of endogenous variables in the VAR
k = 2;     % number of proxies
kn1_number = 4;  % number of sign of structural shock 2
kn2_number = 4;  % number of sign of structural shock 3

mcnumber        = 1e3;  % number of Monte Carlo simulation
numDesireddraws = 1e3;  % number of Desired draws
number          = 1e2;  % number of parfor
conjugateprior  = 'structural';        % when equal to 'none' the volume element is not computed
                                       % when equal to 'structural' the volume element is computed

sigmavalue = [1.6 1.6; 
              5 5];
thetavalue = [0 0;
              0.9 0.9;
              0.9 -0.9];
          
sigman = size(sigmavalue,1);
thetan = size(thetavalue,1);  

trueshocksave = nan(sigman*thetan,mcnumber,T,n+k);
a0save        = nan(sigman*thetan,mcnumber,n+k,n+k,numDesireddraws);
Aplussave     = nan(sigman*thetan,mcnumber,1,n+k,numDesireddraws);
Thetasave     = nan(sigman*thetan,mcnumber,numDesireddraws);
rhosave       = nan(sigman*thetan,mcnumber,numDesireddraws,n);

b1 = 0.5;

Bmatrix = [ 1   b1  b1 ;
            b1  1   b1 ;
            b1  b1  1  ];
        
printi = 100;

delta = [0.3 0.3];

for sigmai = 1:sigman
    for thetai = 1:thetan
        
        theta1 = thetavalue(thetai,1);
        theta2 = thetavalue(thetai,2);
        B1matrix = [ 1  theta1 ;
                     theta2  1 ];
        vsigma1 = sigmavalue(sigmai,1);
        vsigma2 = sigmavalue(sigmai,2);
        vsigma  = [ vsigma1  0;
                    0        vsigma2 ]; 
        
        % VAR Specification
        VAR_Specification

        % Proxy restrictions
        % The zero restrictions will be multiplied by a function
        % F(\tilde{A}_{0},\tilde{A}_{+}) = [\tilde{A}_{0};
        %                                  (\tilde{A}_{0})^{-1\prime}].
        get_proxy_restrictions;

        % Get all zero restrictions under the structural parameterization. That is block restrictions, proxy restrictions, and additional restrictions if any.
        % In this step we collect all the zero restrictions in an array called Zstructural. The zero restrictions will be multiplied by a function
        % F(\tilde{A}_{0},\tilde{A}_{+}) = [\tilde{A}_{0};
        %                                  (\tilde{A}_{0})^{-1\prime};
        %                                   \tilde{A}_{+}].
        get_all_zero_restrictions;
        
        ind1 = (sigmai-1)*thetan+thetai;
        mci = 1;
        while mci <= mcnumber
            
            disp(mci)
            % simulate shock
            u_simulate = randn(T,n);
            v_simulate = randn(T,k);
                        
            % simulate data
            vardata    = u_simulate*Bmatrix';
            % simulate proxy correlated with the first shock
            proxies    = u_simulate(:,n-k+1:n)*B1matrix' + v_simulate*vsigma' + u_simulate(:,1)*delta;
            
            % Estimate
            mainestimate
            
            if doit == 1
                trueshocksave(ind1,mci,:,:) = [u_simulate v_simulate];
                a0save(ind1,mci,:,:,:) = A0tilde;
                Aplussave(ind1,mci,:,:,:) = Aplustilde;
                Thetasave(ind1,mci,:) = Theta;
                rhosave(ind1,mci,:,:) = corrs;
                mci = mci+1;
            end
        end
    end 
end

result_table = nan(n,6);
for sigmai = 1:sigman
    for thetai = 1:thetan
        ind1 = (sigmai-1)*thetan+thetai;
        for j = 1:n
            result_table(j,ind1) = median(mean(rhosave(ind1,:,:,j),2));
        end
    end
end

numString = [];
for ii = 1:11
    numString = [numString '%8.2f & '];
end
numString = [numString '%8.2f'];

FID = fopen(strcat(['correlation_SS',num2str(kn1_number),'.tex']),'w');
fprintf(FID,strcat('%s',numString,'  \\\\  \n'),'$SS_4$ & ',[result_table(2,:),result_table(3,:)]);

%%
save resultSS4 trueshocksave a0save Aplussave Thetasave rhosave T n k mcnumber numDesireddraws sigmavalue thetavalue sigman thetan Bmatrix kn1_number kn2_number

%%
tend = datetime('now');    % end time
computingtime = tend-tstart; % computing time
disp(computingtime)