numdata = [vardata,proxies];

%% Proxy-SVAR Specification
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Write data in model's notation
% \tilde{y}_{t} \tilde{A}0 = \tilde{x}_{t} \tilde{Aplus} + \tilde{constant} + \tilde{e}_{t} for t=1...,T;
% tilde{x}_(t)             = [\tilde{y}_{t-1}, ... , \tilde{y}_{t-p}, constant];
% \tilde_{y_{0:-p+1}       = numdata(1:n_lags,:);
yt  = numdata(n_lags+1:end,:);      % define y transpose as in Rubio-Ramirez, Waggoner, and Zha(2011)
T   = size(yt,1);                   % sample size
xt  = zeros(T,ntilde*n_lags+nex);
for i = 1:n_lags
    xt(:,ntilde*(i-1)+1:ntilde*i) = numdata((n_lags-(i-1)):end-i,:) ;
end
if nex >= 1
    xt(:,ntilde*n_lags+nex)=ones(T,1);
end
Y  = yt;
X  = xt;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Main engine
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute the posterior parameters of the Normal-Generalized-Normal distribution over the structural parameters
SBVAR = SetupSBVAR_Proposal(n_lags, nu, Phi, Psi, Omega_inv, nu_bar, Phi_bar, Psi_bar, Omega_bar_inv, Y, X, Uj, Vj);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set up preliminaries to draw from the triangular-block parameterization and to compute the volume elements

% get the dimensions of (omega_1 and omega_2 as well as W_1 and W_2)
get_dim_Ws_and_info;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Declare objects that will store useful results
A0tilde_save          = nan(ntilde,ntilde,numDesireddraws);   % matrix to store A0tilde
Aplustilde_save       = nan(mtilde,ntilde,numDesireddraws);   % matrix to store Aplustilde
uw_save               = nan(1,numDesireddraws);               % unnormalized importance sampler weights
theta_save            = nan(1,numDesireddraws);

% declare functions to compute the volume elements in both directions
fs       = @(x)ff_h(x,info);
rs       = @(x)ZeroRestrictions(x,info);
fs_inv   = @(x)ff_h_inv(x,info);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize iterations for draws

record = 0;
count  = 0;
doit   = 0;

while doit < 1
    
    if record == 1 && count == 0
        break
    end
    
    A0tilde_draws          = nan(ntilde,ntilde,number);   % matrix to store A0tilde
    Aplustilde_draws       = nan(mtilde,ntilde,number);   % matrix to store Aplustilde
    uw_draws               = nan(1,number);               % unnormalized importance sampler weights
    theta_draws            = nan(1,number);
    int                    = nan(number,1);
    
    parfor pari = 1:number
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Draw \tilde{Llambda}_{0}, \tilde{Llambda}_{+}
        [Llambda0tilde, Llambdaplustilde] = DrawPosterior_SBVAR(SBVAR);
    
        M = Calculate_M(Llambda0tilde,n,k);
    
        SigmatrM = inv(Llambda0tilde)'*M;
        
        if 1
    
            theta  = atan(-SigmatrM(5,2)/SigmatrM(5,3));
            Qstar  = [ 1 0 0 ;
                       0 cos(theta) -sin(theta);
                       0 sin(theta)  cos(theta) ];
            Q      = M(1:n,1:n)*Qstar;
        
            structpara = f_h([reshape(Llambda0tilde,ntilde*ntilde,1);reshape(Llambdaplustilde,mtilde*ntilde,1);...
                              vec(Q); vec(eye(k))],info);

            A0tilde             = reshape(structpara(1:ntilde*ntilde),ntilde,ntilde);
            Aplustilde          = reshape(structpara(1+ntilde*ntilde:end),mtilde,ntilde);
            L0tilde             = inv(A0tilde)';
        
             int(pari)                  = 1;
             A0tilde_draws(:,:,pari)    = A0tilde;
             Aplustilde_draws(:,:,pari) = Aplustilde;
             theta_draws(pari)          = theta;
             
             switch conjugateprior
                 case 'structural'
                     storevefZ_inv     = LogVolumeElement(fs_inv,structpara,rs);
                     log_target        = log_NGN(SBVAR.nu_tilde,SBVAR.Phi_tilde,SBVAR.Psi_tilde,SBVAR.Omega_tilde_inv,reshape(structpara(1:ntilde*ntilde),ntilde,ntilde),reshape(structpara(1+ntilde*ntilde:end),mtilde,ntilde));     
                     log_prop          = log_NGN(SBVAR.nuhat,SBVAR.Phi_hat,SBVAR.Psi_hat,SBVAR.Omega_hat_inv,Llambda0tilde,Llambdaplustilde)  + storevefZ_inv;
                     uw_draws(:,pari)  = exp(log_target- log_prop);
                  case 'none'
                     uw_draws(:,pari) = 1;
             end
        end
    end
    
    intsum = sum(isfinite(int));
    if count+intsum < numDesireddraws
        
        A0tilde_save(:,:,count+1:count+intsum)             = A0tilde_draws(:,:,isfinite(int));
        Aplustilde_save(:,:,count+1:count+intsum)          = Aplustilde_draws(:,:,isfinite(int));
        uw_save(:,count+1:count+intsum)                    = uw_draws(:,isfinite(int));
        theta_save(count+1:count+intsum)                   = theta_draws(isfinite(int));
        
        count = count+intsum;
        
        record = record+1;
        display(['Number of draws = ',num2str(record*number)])
        display(['Number of successful draws = ',num2str(count)])
        display(['Remaining draws = ',num2str(numDesireddraws-(count))])
        
    else
        
        A0tilde_temp            = A0tilde_draws(:,:,isfinite(int));
        Aplustilde_temp         = Aplustilde_draws(:,:,isfinite(int));
        uw_temp                 = uw_draws(:,isfinite(int));
        theta_temp              = theta_draws(isfinite(int));
        
        A0tilde_save(:,:,count+1:numDesireddraws)             = A0tilde_temp(:,:,1:numDesireddraws-count);
        Aplustilde_save(:,:,count+1:numDesireddraws)          = Aplustilde_temp(:,:,1:numDesireddraws-count);
        uw_save(:,count+1:numDesireddraws)                    = uw_temp(:,1:numDesireddraws-count);
        theta_save(count+1:numDesireddraws)                   = theta_temp(1:numDesireddraws-count);
        
        record = record+1;
        display(['Number of draws = ',num2str(record*number)])
        display(['Number of successful draws = ',num2str(numDesireddraws)])
        
        doit = 1;
    end
end

if doit == 1
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % compute the normalized weights and estimate the effective sample size of the importance sampler
    w_IP  = uw_save/sum(uw_save);
    ne    = floor(1/sum(w_IP.^2));

    % useful definitions to store relevant objects
    A0tilde       = zeros(ntilde,ntilde,numDesireddraws);            % define array to store weighted \tilde{A}_0
    Aplustilde    = zeros(ntilde*n_lags+nex,ntilde,numDesireddraws); % define array to store weighted \tilde{A}_plus
    Theta         = nan(numDesireddraws,1);

    % initialize counter to track the state of the importance sampler
    corrs   = nan(numDesireddraws,n);
    parfor s = 1:numDesireddraws
  
        is_draw             = randsample(1:size(w_IP,2),1,true,w_IP);
        A0tilde(:,:,s)      = A0tilde_save(:,:,is_draw);
        Aplustilde(:,:,s)   = Aplustilde_save(:,:,is_draw);
        Theta(s)            = theta_save(is_draw);
        Shocks_identify     = Y*A0tilde_save(:,:,is_draw) -X*Aplustilde_save(:,:,is_draw);
        for ci = 1:n
            temp = corrcoef([Shocks_identify(:,ci) u_simulate(:,ci)]);
            corrs(s,ci) = temp(2,1);
        end      
    end
end