clear variables;
clc;
close all;

addpath('functions'); % set path to helper functions

%% 1. Data: Load the endogenous variables and the proxies
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rawdata = xlsread('data.xlsx','main','B2:BC303');

dates = zeros(304,1);
count = 0;
for i = 1947:2022
    for j = 1:4
        count = count +1;
        dates(count,1) = datenum(i,j*3,1);
    end
end
dates = dates(1:end-2);
% datevec(dates)

dateselect = 1;
switch dateselect
    case 1
        start_date = datenum(1960,3,1);
        end_date   = datenum(2018,6,1);
end

start      = find(dates==start_date);
finish     = find(dates==end_date);

dates      = dates(start:finish,:);
% datevec(dates)

DATASET.TSERIES  = rawdata(start:finish,:);
DATASET.LABEL    = {'dtfp__util','logtfp__util','logPatent','Patent__diff','FFR', ...
                    'GS5','GS10','GDPDEF','News_Instrument1','Nasdaq',   ...    % 10
                    'gz__spread','ebp','default__risk','SP500','GDP', ...
                    'share__c1','share__c2','share__i1','share__i2','Consumption', ...  % 20
                    'Investment','Total hours','NFB hours','Productivity','Amazon__log', ...
                    'Amazon__diff','PRS85006023','CE16OV','CNP16OV','Hours', ...      % 30
                    'GDPDEFg','spread','CPI','realSP500','realSP500p', ...
                    'TFP__AEJM','GDP__AEJM','Investment__AEJM','Consumption__AEJM','Hours__AEJM', ... % 40
                    'SP500__AEJM','Inflation__AEJM','ebp__AEJM','gz__spr__AEJM','defaultrisk__AEJM', ...
                    'tfp__QE','gdp__QE','con__QE','inv__QE','emp__QE', ...                                % 50
                    'NYSE__QE','tfpu__QE','Productivity__QE','dtfp__QE'};
DATASET.MAP      = containers.Map(DATASET.LABEL,1:size(DATASET.TSERIES,2));

var_select_vars  = {'NYSE__QE','Productivity__QE','gdp__QE','con__QE','inv__QE','emp__QE'};
VARData0         = DATASET.TSERIES(:,cell2mat(values(DATASET.MAP,var_select_vars)));
change.log       = [0 0 0 0 0 0];
VARData          = VARData0;
for i = 1:length(var_select_vars)
    if change.log(i) == 1
        VARData(:,i) = log(VARData0(:,i))*100;
    end
end

select_vars      = {'Patent__diff','dtfp__util'};
proxy            = DATASET.TSERIES(:,cell2mat(values(DATASET.MAP,select_vars)));

% VAR specifics
n         = size(VARData,2);
pn        = 12;
result_F  = zeros(pn,n+1);
result_Fr = zeros(pn,n+1);

for p = 4:4     % Lag order  
    
    varEst    = varxest(VARData,ones(size(VARData,1),1),p);
    proxyp    = proxy(p+1:end,:);   
    result_F(p,1)  = p;
    result_Fr(p,1) = p;
    for i = 1:n
        resid     = varEst.U(:,i);
        olsEst    = olsest(proxyp(sum(~isnan(proxyp),2)/2==1,:),resid(sum(~isnan(proxyp),2)/2==1,1),true,true);
        result_F(p,i+1)  = olsEst.F;
        result_Fr(p,i+1) = olsEst.Frobust;
    end
    
end

disp('F test')
disp(result_F)
disp('Robust F test')
disp(result_Fr)
