function SBVAR = SetupSBVAR_Proposal_density_no_lags(n_lags, nu, Phi, nu_bar, Phi_bar,U, n)
%
% The fields available are:
%
%  prior   - 'Conjugate'
%  n_lags  - number of lags
%  ntilde  - number endogenous variables
%  mtilde  - number predetermined variables
%  T       - number observations
%  Y       - endogenous data
%  X       - predetermined data
%  YY      - Y'*Y
%  XX      - X'*X
%  XY      - X'*Y
%  nu_bar            - generalized Gaussian prior parameter
%  Phi_bar_inv       - generalized Gaussian prior parameter
%  mu_bar            - matrix Gaussian prior parameter
%  Omega_bar_inv     - matrix Gaussian prior parameter
%  nu_tilde          - generalized Gaussian posterior parameter
%  Phi_tilde_inv     - generalized Gaussian posterior parameter
%  mu_tilde          - matrix Gaussian posterior parameter
%  Omega_tilde_inv   - matrix Gaussian posterior parameter
%  U     Llambda_0*e{j}    = U{j}*gamma_0{j}
%  V     Llambda_plus*e{j} = V{j}*gamma_+{j} 


% prior type
SBVAR.prior = 'Conjugate';

% sizes
SBVAR.n_lags = n_lags;
SBVAR.ntilde = n;




% unrestricted prior
SBVAR.nu        = nu;
SBVAR.Phi       = Phi;


% unrestricted posterior
SBVAR.nu_tilde =SBVAR.nu;
SBVAR.Phi_tilde =SBVAR.Phi;



% unrestricted proposal prior
SBVAR.nu_bar        = nu_bar;
SBVAR.Phi_bar       = Phi_bar;


% unrestricted proposal posterior 
SBVAR.nu_hat=SBVAR.nu_bar;
SBVAR.Phi_hat=SBVAR.Phi_bar;


% restrictions
SBVAR.U = U;


% restricted posterior proposal
SBVAR.nuhat      = nu_bar;
SBVAR.Sjhat      = cell(SBVAR.ntilde,1);
SBVAR.UThat      = cell(SBVAR.ntilde,1);



for j=1:SBVAR.ntilde
    

    
    Phi_hat_j        = SBVAR.Phi_hat(1+(j-1)*SBVAR.ntilde:j*SBVAR.ntilde,1+(j-1)*SBVAR.ntilde:j*SBVAR.ntilde);

    SBVAR.Sjhat{j}   = (U{j}'*Phi_hat_j*U{j})\eye(size(U{j},2));

    % The following two lines are necessary to draw from the Generalized
    % Normal described in Appendix A.4. See Waggoner and Zha (2003) for
    % details.

    SBVAR.That       = chol(0.5*(SBVAR.Sjhat{j}+SBVAR.Sjhat{j}'))'; 
    SBVAR.UThat{j}   = U{j}*SBVAR.That;   


end

