clear
clc
close all

addpath('functions'); % set path to helper functions
addpath('matlab_720_v3')

rng('default'); % reinitialize the random number generator to its startup configuration
rng(0);         % set seed

tstart = datetime('now');  % start time

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Data: Load the endogenous variables and the proxies
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rawdata = xlsread('data.xlsx','main','B2:BC303');

dates = zeros(304,1);
count = 0;
for i = 1947:2022
    for j = 1:4
        count = count +1;
        dates(count,1) = datenum(i,j*3,1);
    end
end
dates = dates(1:end-2);
% datevec(dates)

dateselect = 1;
switch dateselect
    case 1
        start_date = datenum(1960,3,1);
        end_date   = datenum(2018,6,1);
end

start      = find(dates==start_date);
finish     = find(dates==end_date);

dates      = dates(start:finish,:);
% datevec(dates)

DATASET.TSERIES  = rawdata(start:finish,:);
DATASET.LABEL    = {'dtfp__util','logtfp__util','logPatent','Patent__diff','tfp__QE', ...
                    'gdp__QE','con__QE','inv__QE','emp__QE','NYSE__QE', ...                          % 10
                    'tfpu__QE','Productivity__QE','dtfp__QE'};
DATASET.MAP      = containers.Map(DATASET.LABEL,1:size(DATASET.TSERIES,2));

var_select_vars  = {'gdp__QE','con__QE','inv__QE','emp__QE','NYSE__QE','Productivity__QE'};
VARData          = DATASET.TSERIES(:,cell2mat(values(DATASET.MAP,var_select_vars)));

select_vars      = {'Patent__diff','dtfp__util'};
proxies          = DATASET.TSERIES(:,cell2mat(values(DATASET.MAP,select_vars)));

vardata     = VARData;
numdata     = [vardata,proxies];
varnames    = var_select_vars;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Proxy-SVAR Specification
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% VAR Specification: Data and dimensions
n               = size(vardata,2);     % number of endogenous variables in the VAR
k               = size(proxies,2);     % number of proxies
ntilde          = n + k;               % VAR variables + proxies
n_lags          = 4;                   % number of lags in the VAR
nex             = 1;                   % 1 when constant is included
mtilde          = ntilde*n_lags + nex; % number of predetermined variables
horizons        = 0:0;                 % horizons of IRFs to impose sign restrictions, enters into get_dim_Ws_and_info.m
NS              = 1 + numel(horizons); % number of objects to restrict with proxy or additional zero restrictions, i.e. [\tilde{A}_{0};(\tilde{A}_{0})^{-1\prime}]
Hz              = 48;                  % horizon to compute IRFs
numDesireddraws = 1e3;  % number of Desired draws
number          = 1e5;  % number of parfor
opt_Qdraws      = 5; % No. of initial values of Q in optimisation step of Algorithm 1
opt_jshock      = [n-1;n]; % Index of structural shock of interest

%% Narrative Sign Restrictions
shockNames = cell(1,n);
shockNames{n-1} = 'news';
shockNames{n}   = 'tfp';
NSR = [];

% selection: (1) choose the shocks of the same sign between patent-based
 % and forecast-based news shocks
 %(2) select the periods where the size of the shocks are above average
% 9/1/02	-24.20152718	-0.00434
% 9/1/01	-18.03428317	-0.01246
% 3/1/91	11.64790392	0.004481
% 6/1/95	13.52533824	0.004394
% 9/1/03	14.69182917	0.005607
% 12/1/82	15.27081903	0.00609
% 6/1/91	18.98209774	0.003896
% 3/1/00	40.70833933	0.003699   
news_sign = [2002 9 -1
             2001 9 -1
             1991 3  1
             1995 6  1
             2003 9  1
             1982 12 1
             1991 6  1
             2000 3  1];
       
%Unanticipated tfp shock
% 
%  
% 3/1/96	6.569946357	41.16634751
% 6/1/96	-2.959132275	-42.20802359
% 3/1/98	3.0299908	28.07318542       ** 1.831079866
% 3/1/99	6.709366361	20.47165025
% 6/1/06	-3.176817375	-25.38264355
% 6/1/10	-2.597445504	-29.16143411  ** -1.597455623
% 3/1/12	4.434637681	17.22106822
% 3/1/14	-2.77673057	-39.14347131
% 9/1/15	-2.274369247	-17.58305123   ** -0.640296174

tfp_sign = [1996 3 1
            1996 6 -1
            1998 3  1
            1999 3  1
            2006 6  -1
            2010 6  -1
            2012 3  1
            2014 3 -1
            2015 9 -1];

tfpj = [1:9];
tfpn = length(tfpj);
for i = 1:tfpn
    NSR{i} = {'tfp','sign of shock',datenum(tfp_sign(tfpj(i),1),tfp_sign(tfpj(i),2),01),tfp_sign(tfpj(i),3)};
end

newsj = [1:8];
newsn = length(newsj);
for i = tfpn+1:tfpn+newsn
    NSR{i} = {'news','sign of shock',datenum(news_sign(newsj(i-tfpn),1),news_sign(newsj(i-tfpn),2),01),news_sign(newsj(i-tfpn),3)};
end

tfpjh = [];
tfpnh = length(tfpjh);
for i = tfpn+newsn+1:tfpn+newsn+tfpnh
    NSR{i} = {'tfp','contribution',datenum(tfp_sign(tfpjh(i-tfpn-newsn),1),tfp_sign(tfpjh(i-tfpn-newsn),2),01), ...
               datenum(tfp_sign(tfpjh(i-tfpn-newsn),1),tfp_sign(tfpjh(i-tfpn-newsn),2),01), ...
               'Productivity__QE',1,'weak'};  % strong or weak
end

newsjh = [];
newsnh = length(newsjh);
for i = tfpn+newsn+tfpnh+1:tfpn+newsn+tfpnh+newsnh
    NSR{i} = {'news','contribution',datenum(news_sign(newsjh(i-tfpn-newsn-tfpnh),1),news_sign(newsjh(i-tfpn-newsn-tfpnh),2),01), ...
              datenum(news_sign(newsjh(i-tfpn-newsn-tfpnh),1),news_sign(newsjh(i-tfpn-newsn-tfpnh),2),01), ... 
              'NYSE__QE',1,'weak'};           % strong or weak
end

[restr.Ns,restr.Nc] = get_Narrative_Restrictions(NSR,var_select_vars,dates,shockNames);
Ns                  = restr.Ns(n_lags+1:end,:);
Nc                  = restr.Nc(n_lags+1:end,n_lags+1:end,:,:);

findnarrativecontrib = find(Nc~=0);
[startperiod,~,shock,variable] = ind2sub(size(Nc),find(Nc~=0));

% Write data in model's notation
% \tilde{y}_{t} \tilde{A}0 = \tilde{x}_{t} \tilde{Aplus} + \tilde{constant} + \tilde{e}_{t} for t=1...,T;
% tilde{x}_(t)             = [\tilde{y}_{t-1}, ... , \tilde{y}_{t-p}, constant];
% \tilde_{y_{0:-p+1}       = numdata(1:n_lags,:);
yt  = numdata(n_lags+1:end,:);      % define y transpose as in Rubio-Ramirez, Waggoner, and Zha(2011)
T   = size(yt,1);                   % sample size
xt  = zeros(T,ntilde*n_lags+nex);
for i = 1:n_lags
    xt(:,ntilde*(i-1)+1:ntilde*i) = numdata((n_lags-(i-1)):end-i,:) ;
end
if nex >= 1
    xt(:,ntilde*n_lags+nex)=ones(T,1);
end
Y  = yt;
X  = xt;

% SVAR Priors 
nu            = ntilde;                                          % degrees of freedom
Phi           = eye(ntilde^2)*1e-16;                             % Phi^-1: variance of \tilde{A}_0.
Psi           = zeros((n_lags*ntilde + nex)*ntilde,ntilde^2);    % Psi: mean of \tilde{A}_+
Omega_inv     = eye((n_lags*ntilde + nex)*ntilde)*1e-16;         % Omega: variance of \tilde{A}_+

% Prior for the Proposal
nu_bar        = ntilde;                                       % degrees of freedom
Phi_bar       = eye(ntilde^2)*1e-16;                          % Phi^-1:  variance of \tilde{A}_0.
Psi_bar       = zeros((n_lags*ntilde + nex)*ntilde,ntilde^2); % Psi_bar: mean of \tilde{A}_+
Omega_bar_inv = eye((n_lags*ntilde + nex)*ntilde)*1e-16;      % Omega:   variance of \tilde{A}_+

% Define Uj and Vj matrices. See Appendix A.4.
Uj = cell([ntilde,1]);
for i = 1:ntilde
    Uj{i} = eye([ntilde,i]); 
end
Vj = cell([ntilde,1]);
for i = 1:n
    Vj{i} = zeros(mtilde,mtilde-n_lags*k);
    for ell = 1:n_lags
        Vj{i}(1 + (ell-1)*ntilde:ell*ntilde,1 + (ell-1)*n:n + (ell-1)*n) = eye([ntilde,n]);
    end
    if nex > 0
        Vj{i}(mtilde-nex+1:mtilde,n_lags*n+1:n_lags*n+nex)= eye(nex);
    end
end
for i = n+1:ntilde
    Vj{i} = eye(mtilde);
end

% Store the dimensions of the rows of U and the columns of V. Input for get_dim_Ws_and_info.m
WW = cell([ntilde,1]);
for i=1:ntilde
     WW{i} = zeros(size(Vj{i},1),size(Uj{i},2));
end

% Proxy restrictions
% The zero restrictions will be multiplied by a function
% F(\tilde{A}_{0},\tilde{A}_{+}) = [\tilde{A}_{0};
%                                  (\tilde{A}_{0})^{-1\prime}].
get_proxy_restrictions;

% Get all zero restrictions under the structural parameterization. That is block restrictions, proxy restrictions, and additional restrictions if any.
% In this step we collect all the zero restrictions in an array called Zstructural. The zero restrictions will be multiplied by a function
% F(\tilde{A}_{0},\tilde{A}_{+}) = [\tilde{A}_{0};
%                                  (\tilde{A}_{0})^{-1\prime};
%                                   \tilde{A}_{+}].
get_all_zero_restrictions;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Main engine
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute the posterior parameters of the Normal-Generalized-Normal distribution over the structural parameters
SBVAR = SetupSBVAR_Proposal(n_lags, nu, Phi, Psi, Omega_inv, nu_bar, Phi_bar, Psi_bar, Omega_bar_inv, Y, X, Uj, Vj);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set up preliminaries to draw from the triangular-block parameterization and to compute the volume elements

% get the dimensions of (omega_1 and omega_2 as well as W_1 and W_2)
get_dim_Ws_and_info;

% declare functions to compute the volume elements in both directions
fs = @(x)ff_h_A(x,info);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Declare objects that will store useful results
Llambda0tilde_save    = nan(ntilde,ntilde,numDesireddraws);   % matrix to store Llambda0tilde
Llambdaplustilde_save = nan(mtilde,ntilde,numDesireddraws);   % matrix to store Llambdaplustilde
Q_save                = nan(ntilde,ntilde,opt_Qdraws,numDesireddraws);
initialnumber_save    = nan(numDesireddraws,1);
rMinPost1             = nan(Hz,n,numDesireddraws);
rMaxPost1             = rMinPost1;
rMinPost2             = rMinPost1;
rMaxPost2             = rMinPost1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
restrf = zeros(n,1);
restrf(1:n-k) = k;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize iterations for draws

drawnumber = 0;
count = 0;
while count < numDesireddraws
    
    drawnumber = drawnumber + 1;
    disp(strcat('drawnumber = ',num2str(drawnumber)))
    disp(strcat('count = ',num2str(count)))
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Draw \tilde{Llambda}_{0}, \tilde{Llambda}_{+}
    checkStable = 0;
    while checkStable < 1
        [Llambda0tilde,Llambdaplustilde] = DrawPosterior_SBVAR(SBVAR);
        checkStable = VAReigenvalueCheck(Llambda0tilde, Llambdaplustilde,info);
    end
    
    % Zero Restrictions
    structpara0 = f_h([reshape(Llambda0tilde,ntilde*ntilde,1);reshape(Llambdaplustilde,mtilde*ntilde,1);vec(eye(n));vec(eye(k))],info);
    restrF0     = info.ZF(structpara0,info);
    restrF      = kron(ones(n-2,1),restrF0{1});
    restrF      = restrF(:,1:n);
    
    % Sign normalization restriction
    SStd = zeros(k,n^2);
    temp = inv(Llambda0tilde)';
    SStd(1,(n-2)*n+1:(n-1)*n) = temp(end-1,1:n);
    SStd(2,(n-1)*n+1:n^2) = temp(end,1:n);
    
    % Narrative Sign restriction
    SSNs = zeros(tfpn+newsn,n^2);
    epsilon0 = Y*Llambda0tilde - X*Llambdaplustilde;
    ind1 = Ns(:,n-1) ~= 0;
    SSNs(1:newsn,(n-2)*n+1:(n-1)*n) = epsilon0(ind1,1:n).*kron(Ns(ind1,n-1),ones(1,n));
    ind2 = Ns(:,n) ~= 0;
    SSNs(newsn+1:end,(n-1)*n+1:n^2) = epsilon0(ind2,1:n).*kron(Ns(ind2,n),ones(1,n));
        
    Qtilde0_draws = nan(ntilde,ntilde,number); 
    int     = nan(number,1);
    parfor i = 1:number
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Impose the zero restrictions on \tilde{Lambda}_{0} and (\tilde{Lambda}_{0}')^{-1}, i.e. on F(\tilde{Lambda}_{0},\tilde{Lambda}_{+})
        w1 = DrawW(n,dim1,W1);
        w2 = DrawW(k,dim2,W2);
        [structpara,Qtilde0] = fs([reshape(Llambda0tilde,ntilde*ntilde,1);reshape(Llambdaplustilde,mtilde*ntilde,1);w1;w2]);

        A0tilde    = reshape(structpara(1:ntilde*ntilde),ntilde,ntilde);
        Aplustilde = reshape(structpara(1+ntilde*ntilde:end),mtilde,ntilde);
        V          = ([zeros(k,n),eye(k)]*(inv(A0tilde)')*[eye(n),zeros(n,k)]')*([zeros(k,n-k),eye(k)]');
        % Increase efficiency
        if V(1,1) < 0 
            A0tilde(:,n-1) = -A0tilde(:,n-1);
            Aplustilde(:,n-1) = -Aplustilde(:,n-1);
            Qtilde0(:,n-1) = -Qtilde0(:,n-1);
        end
        if V(2,2) < 0
            A0tilde(:,n) = -A0tilde(:,n);
            Aplustilde(:,n) = -Aplustilde(:,n);
            Qtilde0(:,n) = -Qtilde0(:,n);
        end
        V = ([zeros(k,n),eye(k)]*(inv(A0tilde)')*[eye(n),zeros(n,k)]')*([zeros(k,n-k),eye(k)]');
        
        % Shock sign restrictions
        Shocks_draw         = Y*A0tilde - X*Aplustilde;
        epsilon0            = Shocks_draw';
        % +*1=+, -*(-1)=+ 0*=0    
        checkss = min([epsilon0(n-1,:)'.* Ns(:,n-1); epsilon0(n,:)'.* Ns(:,n)]) == 0;
        
        if V(1,1)>0 && V(2,2)>0 && checkss 
            int(i) = 1;
            Qtilde0_draws(:,:,i) = Qtilde0;
        end
    end
    
    intsum = sum(isfinite(int));
    if intsum > 0
        indices = find(isfinite(int) == 1);
        Q0 = Qtilde0_draws(:,:,indices(1:min(intsum,opt_Qdraws)));
        [rmin_draws1,rmax_draws1,flag1] = numericalBoundsPar(restrF,restrf,SStd,SSNs,Q0,Llambda0tilde,Llambdaplustilde,Hz,opt_jshock(1),info);
        [rmin_draws2,rmax_draws2,flag2] = numericalBoundsPar(restrF,restrf,SStd,SSNs,Q0,Llambda0tilde,Llambdaplustilde,Hz,opt_jshock(2),info);
        if flag1 == 1 && flag2 == 1
            count = count +1;
            initialnumber_save(count) = intsum;
            Llambda0tilde_save(:,:,count) = Llambda0tilde;
            Llambdaplustilde_save(:,:,count) = Llambdaplustilde;
            Q_save(:,:,1:min(intsum,opt_Qdraws),count) = Q0;
            rMinPost1(:,:,count) = rmin_draws1;
            rMaxPost1(:,:,count) = rmax_draws1;
            rMinPost2(:,:,count) = rmin_draws2;
            rMaxPost2(:,:,count) = rmax_draws2;
        end
    end
end

% Compute posterior mean under single prior.
rSinglePriorPost1 = nan(Hz,n,numDesireddraws);
rSinglePriorPost2 = rSinglePriorPost1;
for i = 1:numDesireddraws
    structpara0 = f_h([reshape(Llambda0tilde_save(:,:,i),ntilde*ntilde,1); ...
                  reshape(Llambdaplustilde_save(:,:,i),mtilde*ntilde,1); ... 
                  vec(Q_save(1:n,1:n,1,i));vec(Q_save(n+1:end,n+1:end,1,i))],info);
    A0tilde0    = reshape(structpara0(1:ntilde*ntilde),ntilde,ntilde);
    Aplustilde0 = reshape(structpara0(1+ntilde*ntilde:end),mtilde,ntilde);
    
    CT = [inv(A0tilde0)'*Aplustilde0(1:end-1,:)';eye((n_lags-1)*ntilde,n_lags*ntilde)];
    AT = [inv(A0tilde0)';zeros(ntilde*n_lags-ntilde,ntilde)]; 
    vma = nan(ntilde,ntilde,Hz);
    for h = 1:Hz
        irf = (CT^(h-1))*AT;
        vma(:,:,h) = irf(1:ntilde,:);
    end
    rSinglePriorPost1(:,:,i) = squeeze(vma(1:n,opt_jshock(1),:))';
    rSinglePriorPost2(:,:,i) = squeeze(vma(1:n,opt_jshock(2),:))';
end

%% news shock
rSinglePriorPost = rSinglePriorPost1;
rMinPost = rMinPost1;
rMaxPost = rMaxPost1;
plot_robust_IRF

%% tfp shock
rSinglePriorPost = rSinglePriorPost2;
rMinPost = rMinPost2;
rMaxPost = rMaxPost2;
plot_robust_IRF

%% Save Result
save result

%%
tend = datetime('now');    % end time
computingtime = tend-tstart; % computing time
disp(computingtime)