clear
clc
close all

addpath('functions'); % set path to helper functions
addpath('matlab_720_v3')

rng('default'); % reinitialize the random number generator to its startup configuration
rng(0);         % set seed

tstart = datetime('now');  % start time

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Data: Load the endogenous variables and the proxies
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rawdata = xlsread('data.xlsx','main','B2:N303');

dates = zeros(304,1);
count = 0;
for i = 1947:2022
    for j = 1:4
        count = count +1;
        dates(count,1) = datenum(i,j*3,1);
    end
end
dates = dates(1:end-2);
% datevec(dates)

dateselect = 1;
switch dateselect
    case 1
        start_date = datenum(1960,3,1);
        end_date   = datenum(2018,6,1);
end

start      = find(dates==start_date);
finish     = find(dates==end_date);

dates      = dates(start:finish,:);
% datevec(dates)

DATASET.TSERIES  = rawdata(start:finish,:);
DATASET.LABEL    = {'dtfp__util','logtfp__util','logPatent','Patent__diff','tfp__QE', ...
                    'gdp__QE','con__QE','inv__QE','emp__QE','NYSE__QE', ...                          % 10
                    'tfpu__QE','Productivity__QE','dtfp__QE'};
DATASET.MAP      = containers.Map(DATASET.LABEL,1:size(DATASET.TSERIES,2));

var_select_vars  = {'gdp__QE','con__QE','inv__QE','emp__QE','NYSE__QE','Productivity__QE'};
VARData          = DATASET.TSERIES(:,cell2mat(values(DATASET.MAP,var_select_vars)));

select_vars      = {'Patent__diff','dtfp__util'};
proxies          = DATASET.TSERIES(:,cell2mat(values(DATASET.MAP,select_vars)));

vardata     = VARData;
numdata     = [vardata,proxies];
varnames    = var_select_vars;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Proxy-SVAR Specification
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% VAR Specification: Data and dimensions
n               = size(vardata,2);     % number of endogenous variables in the VAR
k               = size(proxies,2);     % number of proxies
ntilde          = n + k;               % VAR variables + proxies
n_lags          = 4;                   % number of lags in the VAR
nex             = 1;                   % 1 when constant is included
mtilde          = ntilde*n_lags + nex; % number of predetermined variables
horizons        = 0:0;                 % horizons of IRFs to impose sign restrictions, enters into get_dim_Ws_and_info.m
NS              = 1 + numel(horizons); % number of objects to restrict with proxy or additional zero restrictions, i.e. [\tilde{A}_{0};(\tilde{A}_{0})^{-1\prime}]
Hz              = 48;                  % horizon to compute IRFs
conjugateprior  = 'structural';        % when equal to 'none' the volume element is not computed
numDesireddraws = 5e5;  % number of Desired draws
number          = 1e4;  % number of parfor
narra_weight    = 0;    % 1: calculate the weight induced by narrative sign restrictions
nDrawWeights    = 1e4;
ZeroR           = 0;   % 1: impose zero restriciton
uniform_prior   = 1;

%% Narrative Sign Restrictions
shockNames = cell(1,n);
shockNames{n-1} = 'news';
shockNames{n}   = 'tfp';
NSR = [];

% selection: (1) choose the shocks of the same sign between patent-based
 % and forecast-based news shocks
 %(2) select the periods where the size of the shocks are above average
% 9/1/02	-24.20152718	-0.00434
% 9/1/01	-18.03428317	-0.01246
% 3/1/91	11.64790392	0.004481
% 6/1/95	13.52533824	0.004394
% 9/1/03	14.69182917	0.005607
% 12/1/82	15.27081903	0.00609
% 6/1/91	18.98209774	0.003896
% 3/1/00	40.70833933	0.003699   
news_sign = [2002 9 -1
             2001 9 -1
             1991 3  1
             1995 6  1
             2003 9  1
             1982 12 1
             1991 6  1
             2000 3  1];
       
%Unanticipated tfp shock
% 
%  
% 3/1/96	6.569946357	41.16634751
% 6/1/96	-2.959132275	-42.20802359
% 3/1/98	3.0299908	28.07318542       ** 1.831079866
% 3/1/99	6.709366361	20.47165025
% 6/1/06	-3.176817375	-25.38264355
% 6/1/10	-2.597445504	-29.16143411  ** -1.597455623
% 3/1/12	4.434637681	17.22106822
% 3/1/14	-2.77673057	-39.14347131
% 9/1/15	-2.274369247	-17.58305123   ** -0.640296174

tfp_sign = [1996 3 1
            1996 6 -1
            1998 3  1
            1999 3  1
            2006 6  -1
            2010 6  -1
            2012 3  1
            2014 3 -1
            2015 9 -1];

tfpj = [  1:9 ];
tfpn = length(tfpj);
for i = 1:tfpn
    NSR{i} = {'tfp','sign of shock',datenum(tfp_sign(tfpj(i),1),tfp_sign(tfpj(i),2),01),tfp_sign(tfpj(i),3)};
end

newsj = [ 1:8 ];
newsn = length(newsj);
for i = tfpn+1:tfpn+newsn
    NSR{i} = {'news','sign of shock',datenum(news_sign(newsj(i-tfpn),1),news_sign(newsj(i-tfpn),2),01),news_sign(newsj(i-tfpn),3)};
end

tfpjh = [];
tfpnh = length(tfpjh);
for i = tfpn+newsn+1:tfpn+newsn+tfpnh
    NSR{i} = {'tfp','contribution',datenum(tfp_sign(tfpjh(i-tfpn-newsn),1),tfp_sign(tfpjh(i-tfpn-newsn),2),01), ...
               datenum(tfp_sign(tfpjh(i-tfpn-newsn),1),tfp_sign(tfpjh(i-tfpn-newsn),2),01), ...
               'Productivity__QE',1,'weak'};  % strong or weak
end

newsjh = [];
newsnh = length(newsjh);
for i = tfpn+newsn+tfpnh+1:tfpn+newsn+tfpnh+newsnh
    NSR{i} = {'news','contribution',datenum(news_sign(newsjh(i-tfpn-newsn-tfpnh),1),news_sign(newsjh(i-tfpn-newsn-tfpnh),2),01), ...
              datenum(news_sign(newsjh(i-tfpn-newsn-tfpnh),1),news_sign(newsjh(i-tfpn-newsn-tfpnh),2),01), ... 
              'NYSE__QE',1,'weak'};           % strong or weak
end

[restr.Ns,restr.Nc] = get_Narrative_Restrictions(NSR,var_select_vars,dates,shockNames);
Ns                  = restr.Ns(n_lags+1:end,:);
Nc                  = restr.Nc(n_lags+1:end,n_lags+1:end,:,:);

findnarrativecontrib = find(Nc~=0);
[startperiod,~,shock,variable] = ind2sub(size(Nc),find(Nc~=0));

% Write data in model's notation
% \tilde{y}_{t} \tilde{A}0 = \tilde{x}_{t} \tilde{Aplus} + \tilde{constant} + \tilde{e}_{t} for t=1...,T;
% tilde{x}_(t)             = [\tilde{y}_{t-1}, ... , \tilde{y}_{t-p}, constant];
% \tilde_{y_{0:-p+1}       = numdata(1:n_lags,:);
yt  = numdata(n_lags+1:end,:);      % define y transpose as in Rubio-Ramirez, Waggoner, and Zha(2011)
T   = size(yt,1);                   % sample size
xt  = zeros(T,ntilde*n_lags+nex);
for i = 1:n_lags
    xt(:,ntilde*(i-1)+1:ntilde*i) = numdata((n_lags-(i-1)):end-i,:) ;
end
if nex >= 1
    xt(:,ntilde*n_lags+nex)=ones(T,1);
end
Y  = yt;
X  = xt;

% SVAR Priors 
nu            = ntilde;                                      % degrees of freedom
Phi           = zeros(ntilde^2);                             % Phi^-1: variance of \tilde{A}_0.
Psi           = zeros(mtilde*ntilde,ntilde^2);               % Psi: mean of \tilde{A}_+
Omega_inv     = zeros(mtilde*ntilde);                        % Omega: variance of \tilde{A}_+

% parameters for the Proposal distribution
% the parameters are chosen to maximize the effective sample size of the importance sampler.
load('parameters_Proposal.mat')

hi = 1;

nu_bar = para_save(1,hi);   % -ntilde*2-2
MinnesotaPrior_Psi = zeros(mtilde,ntilde);
diag_Phi = para_save(2:ntilde+1,hi);
MinnesotaPrior_Phi = diag(diag_Phi);
diag_Omega_inv = para_save(ntilde+2:1+ntilde+mtilde,hi);
MinnesotaPrior_Omega_inv = diag(diag_Omega_inv);

Phi_bar       = kron(eye(ntilde),MinnesotaPrior_Phi);         % Phi^-1:  variance of \tilde{A}_0.
Psi_bar       = kron(eye(ntilde),MinnesotaPrior_Psi);         % Psi_bar: mean of \tilde{A}_+
Omega_bar_inv = kron(eye(ntilde),MinnesotaPrior_Omega_inv);    % Omega:   variance of \tilde{A}_+

% Define Uj and Vj matrices. See Appendix A.4.
Uj = cell([ntilde,1]);
for i = 1:ntilde
    Uj{i} = eye([ntilde,i]); 
end
Vj = cell([ntilde,1]);
for i = 1:n
    Vj{i} = zeros(mtilde,mtilde-n_lags*k);
    for ell = 1:n_lags
        Vj{i}(1 + (ell-1)*ntilde:ell*ntilde,1 + (ell-1)*n:n + (ell-1)*n) = eye([ntilde,n]);
    end
    if nex > 0
        Vj{i}(mtilde-nex+1:mtilde,n_lags*n+1:n_lags*n+nex)= eye(nex);
    end
end
for i = n+1:ntilde
    Vj{i} = eye(mtilde);
end

% Store the dimensions of the rows of U and the columns of V. Input for get_dim_Ws_and_info.m
WW = cell([ntilde,1]);
for i=1:ntilde
     WW{i} = zeros(size(Vj{i},1),size(Uj{i},2));
end

% Proxy restrictions
% The zero restrictions will be multiplied by a function
% F(\tilde{A}_{0},\tilde{A}_{+}) = [\tilde{A}_{0};
%                                  (\tilde{A}_{0})^{-1\prime}].
get_proxy_restrictions;

% Get all zero restrictions under the structural parameterization. That is block restrictions, proxy restrictions, and additional restrictions if any.
% In this step we collect all the zero restrictions in an array called Zstructural. The zero restrictions will be multiplied by a function
% F(\tilde{A}_{0},\tilde{A}_{+}) = [\tilde{A}_{0};
%                                  (\tilde{A}_{0})^{-1\prime};
%                                   \tilde{A}_{+}].
get_all_zero_restrictions;

% Impose additional restrictions 
if ZeroR == 1
    Z{n-1}(rank(Z{n-1})+1,ntilde+n+k) = 1;                       
    Zstructural{n-1}(rank(Zstructural{n-1})+1,ntilde+n+k) = 1;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Main engine
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute the posterior parameters of the Normal-Generalized-Normal distribution over the structural parameters
SBVAR = SetupSBVAR_Proposal(n_lags, nu, Phi, Psi, Omega_inv, nu_bar, Phi_bar, Psi_bar, Omega_bar_inv, Y, X, Uj, Vj);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set up preliminaries to draw from the triangular-block parameterization and to compute the volume elements

% get the dimensions of (omega_1 and omega_2 as well as W_1 and W_2)
get_dim_Ws_and_info;

% declare functions to compute the volume elements in both directions
fs       = @(x)ff_h(x,info);
rs       = @(x)ZeroRestrictions(x,info);
fs_inv   = @(x)ff_h_inv(x,info);

if uniform_prior == 1
    fo_str2irfs     = @(x)StructuralToIRF(x,info);
    fo_str2irfs_inv = @(x)IRFToStructural(x,info);
    r_irfs          = @(x)IRFRestrictions(x,info); 
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Declare objects that will store useful results
A0tilde_save          = nan(ntilde,ntilde,numDesireddraws);   % matrix to store A0tilde
Aplustilde_save       = nan(mtilde,ntilde,numDesireddraws);   % matrix to store Aplustilde
Llambda0tilde_save    = nan(ntilde,ntilde,numDesireddraws);   % matrix to store Llambda0tilde
Llambdaplustilde_save = nan(mtilde,ntilde,numDesireddraws);   % matrix to store Llambdaplustilde
mv_save               = nan(1,numDesireddraws);               % store minimum eigenvalue of the Reliability matrix
uw_save               = nan(4,numDesireddraws);               % unnormalized importance sampler weights
uw_uniform_save       = nan(4,numDesireddraws);               % unnormalized importance sampler weights using a uniform joint prior distribution

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize iterations for draws

doit   = 0;
record = 0;
count  = 0;

while doit < 1
    
    A0tilde_draws          = nan(ntilde,ntilde,number);   % matrix to store A0tilde
    Aplustilde_draws       = nan(mtilde,ntilde,number);   % matrix to store Aplustilde
    Llambda0tilde_draws    = nan(ntilde,ntilde,number);   % matrix to store Llambda0tilde
    Llambdaplustilde_draws = nan(mtilde,ntilde,number);   % matrix to store Llambdaplustilde
    mv_draws               = nan(1,number);               % store minimum eigenvalue of the Reliability matrix
    uw_draws               = nan(4,number);               % unnormalized importance sampler weights
    uw_uniform_draws       = nan(4,number);               % unnormalized importance sampler weights using a uniform joint prior distribution
    int                    = nan(number,1);
    
    parfor i = 1:number
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Draw \tilde{Llambda}_{0}, \tilde{Llambda}_{+}
        checkStable = 0;
        while checkStable < 1
            [Llambda0tilde, Llambdaplustilde] = DrawPosterior_SBVAR(SBVAR);
            if n_lags == 0
                checkStable = 1;
            else
                checkStable = VAReigenvalueCheck(Llambda0tilde, Llambdaplustilde,info);
            end
        end
    
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Impose the zero restrictions on \tilde{Lambda}_{0} and (\tilde{Lambda}_{0}')^{-1}, i.e. on F(\tilde{Lambda}_{0},\tilde{Lambda}_{+})
        w1         = DrawW(n,dim1,W1);
        w2         = DrawW(k,dim2,W2);
        structpara = fs([reshape(Llambda0tilde,ntilde*ntilde,1);reshape(Llambdaplustilde,mtilde*ntilde,1);w1;w2]);

        A0tilde    = reshape(structpara(1:ntilde*ntilde),ntilde,ntilde);
        % Aplustilde = reshape(structpara(1+ntilde*ntilde:end),mtilde,ntilde);
        V          = ([zeros(k,n),eye(k)]*(inv(A0tilde)')*[eye(n),zeros(n,k)]')*([zeros(k,n-k),eye(k)]');
        % Increase efficiency
        if V(1,1) < 0 
            structpara(ntilde*(n-2)+1:ntilde*(n-1)) = - structpara(ntilde*(n-2)+1:ntilde*(n-1));
            structpara(ntilde*ntilde+mtilde*(n-2)+1:ntilde*ntilde+mtilde*(n-1)) = - structpara(ntilde*ntilde+mtilde*(n-2)+1:ntilde*ntilde+mtilde*(n-1));
            % A0tilde(:,n-1) = -A0tilde(:,n-1);
            % Aplustilde(:,n-1) = -Aplustilde(:,n-1);
        end
        if V(2,2) < 0
            structpara(ntilde*(n-1)+1:ntilde*n) = - structpara(ntilde*(n-1)+1:ntilde*n);
            structpara(ntilde*ntilde+mtilde*(n-1)+1:ntilde*ntilde+mtilde*n) = - structpara(ntilde*ntilde+mtilde*(n-1)+1:ntilde*ntilde+mtilde*n);
            % A0tilde(:,n) = -A0tilde(:,n);
            % Aplustilde(:,n) = -Aplustilde(:,n);
        end
        A0tilde    = reshape(structpara(1:ntilde*ntilde),ntilde,ntilde);
        Aplustilde = reshape(structpara(1+ntilde*ntilde:end),mtilde,ntilde);
        V                   = ([zeros(k,n),eye(k)]*(inv(A0tilde)')*[eye(n),zeros(n,k)]')*([zeros(k,n-k),eye(k)]');
        G_02                = A0tilde(n+1:n+k,n+1:n+k);
        reliability_matrix  = (inv(G_02)'*inv(G_02) + V*V')\(V*V');
        min_ev_RM           = min(eig(reliability_matrix));
        L0tilde             = inv(A0tilde)';
        
        % Shock sign restrictions
        Shocks_draw         = Y*A0tilde - X*Aplustilde;
        epsilon0            = Shocks_draw';
        % +*1=+, -*(-1)=+ 0*=0    
        checkss = min([epsilon0(n-1,:)'.* Ns(:,n-1); epsilon0(n,:)'.* Ns(:,n)]) == 0;
        
        % Shock historical decomposition restriction 
        check_narrative_contrib = zeros(length(findnarrativecontrib),1);
        for restrictionhd = 1:length(findnarrativecontrib)
            HD = L0tilde(variable(restrictionhd),:)'.*epsilon0(:,startperiod(restrictionhd));
            switch Nc(findnarrativecontrib(restrictionhd))
                case 1
                    check_narrative_contrib(restrictionhd) = abs(HD(shock(restrictionhd))) == max(abs(HD));
                case 2
                    check_narrative_contrib(restrictionhd) = abs(HD(shock(restrictionhd))) > sum(abs(HD(1:end ~= (shock(restrictionhd)))));
            end
        end
        checkhd = (sum(check_narrative_contrib) == length(findnarrativecontrib));
        
        if V(1,1)>0 && V(2,2)>0 && checkss && checkhd 
             
             int(i) = 1;
             A0tilde_draws(:,:,i)            = A0tilde;
             Aplustilde_draws(:,:,i)         = Aplustilde;
             Llambda0tilde_draws(:,:,i)      = Llambda0tilde;
             Llambdaplustilde_draws(:,:,i)   = Llambdaplustilde;
             mv_draws(:,i)                   = min_ev_RM;
             
            %% compute importance sampling weights
             if narra_weight == 1
                 check_simulated = nan(nDrawWeights,1);
                 for rep = 1:nDrawWeights
                     for restrictionhd = 1:length(findnarrativecontrib)
                         HD = L0tilde(variable(restrictionhd),:)'.*randn(ntilde,1);
                         switch Nc(findnarrativecontrib(restrictionhd))
                             case 1
                                 check_narrative_contrib(restrictionhd) = abs(HD(shock(restrictionhd))) == max(abs(HD));
                             case 2
                                 check_narrative_contrib(restrictionhd) = abs(HD(shock(restrictionhd))) > sum(abs(HD(1:end ~= (shock(restrictionhd)))));
                         end
                     end
                     check_simulated(rep) =  (sum(check_narrative_contrib) == length(findnarrativecontrib));
                 end
                 weighthd = nDrawWeights/(sum(check_simulated));
             else
                 weighthd = 1;
             end
        
            %% compute importance sampling weights
             switch conjugateprior    
                 case 'structural'
                     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                     storevefZ_inv           = LogVolumeElement(fs_inv,structpara,rs);
                     log_target              = log_NGN(SBVAR.nu_tilde,SBVAR.Phi_tilde,SBVAR.Psi_tilde,SBVAR.Omega_tilde_inv,A0tilde,Aplustilde);     
                     log_prop                = log_NGN(SBVAR.nuhat,SBVAR.Phi_hat,SBVAR.Psi_hat,SBVAR.Omega_hat_inv,Llambda0tilde,Llambdaplustilde)  + storevefZ_inv;
                     uw_draws(:,i)           = [ exp(log_target- log_prop)*weighthd; ...
                                                 log_NGN(SBVAR.nu_tilde,SBVAR.Phi_tilde,SBVAR.Psi_tilde,SBVAR.Omega_tilde_inv,A0tilde,Aplustilde); ...
                                                 log_NGN(SBVAR.nuhat,SBVAR.Phi_hat,SBVAR.Psi_hat,SBVAR.Omega_hat_inv,Llambda0tilde,Llambdaplustilde); ...
                                                 storevefZ_inv ];
                 
                 case 'none'
                    uw_draws(:,i)           = 1;
             end
             
             if uniform_prior == 1
                 irfpara          = fo_str2irfs(structpara);
                 storevegphiZ_inv = LogVolumeElement(fs_inv,structpara,rs) + LogVolumeElement(fo_str2irfs_inv,irfpara,r_irfs);
                 log_target       = log_NGN(SBVAR.nu_tilde,SBVAR.Phi_tilde,SBVAR.Psi_tilde,SBVAR.Omega_tilde_inv,Llambda0tilde,Llambdaplustilde);
                 log_prop         = log_NGN(SBVAR.nuhat,SBVAR.Phi_hat,SBVAR.Psi_hat,SBVAR.Omega_hat_inv,Llambda0tilde,Llambdaplustilde)  + storevegphiZ_inv;
                 uw_uniform_draws(:,i) = [exp(log_target- log_prop); ...
                                          log_NGN(SBVAR.nu_tilde,SBVAR.Phi_tilde,SBVAR.Psi_tilde,SBVAR.Omega_tilde_inv,Llambda0tilde,Llambdaplustilde); ...
                                          log_NGN(SBVAR.nuhat,SBVAR.Phi_hat,SBVAR.Psi_hat,SBVAR.Omega_hat_inv,Llambda0tilde,Llambdaplustilde); ...
                                          storevegphiZ_inv ];
             end
             
         end
    end
    
    intsum = sum(isfinite(int));
    if count+intsum < numDesireddraws
        
        A0tilde_save(:,:,count+1:count+intsum)             = A0tilde_draws(:,:,isfinite(int));
        Aplustilde_save(:,:,count+1:count+intsum)          = Aplustilde_draws(:,:,isfinite(int));
        Llambda0tilde_save(:,:,count+1:count+intsum)       = Llambda0tilde_draws(:,:,isfinite(int));
        Llambdaplustilde_save(:,:,count+1:count+intsum)    = Llambdaplustilde_draws(:,:,isfinite(int));
        mv_save(:,count+1:count+intsum)                    = mv_draws(:,isfinite(int));
        uw_save(:,count+1:count+intsum)                    = uw_draws(:,isfinite(int));
        uw_uniform_save(:,count+1:count+intsum)            = uw_uniform_draws(:,isfinite(int));
        
        count = count+intsum;
        
        record = record+1;
        display(['Number of draws = ',num2str(record*number)])
        display(['Number of successful draws = ',num2str(count)])
        display(['Remaining draws = ',num2str(numDesireddraws-(count))])
        
    else
        
        A0tilde_temp            = A0tilde_draws(:,:,isfinite(int));
        Aplustilde_temp         = Aplustilde_draws(:,:,isfinite(int));
        Llambda0tilde_temp      = Llambda0tilde_draws(:,:,isfinite(int));
        Llambdaplustilde_temp   = Llambdaplustilde_draws(:,:,isfinite(int));
        mv_temp                 = mv_draws(:,isfinite(int));
        uw_temp                 = uw_draws(:,isfinite(int));
        uw_uniform_temp         = uw_uniform_draws(:,isfinite(int));
        
        A0tilde_save(:,:,count+1:numDesireddraws)             = A0tilde_temp(:,:,1:numDesireddraws-count);
        Aplustilde_save(:,:,count+1:numDesireddraws)          = Aplustilde_temp(:,:,1:numDesireddraws-count);
        Llambda0tilde_save(:,:,count+1:numDesireddraws)       = Llambda0tilde_temp(:,:,1:numDesireddraws-count);
        Llambdaplustilde_save(:,:,count+1:numDesireddraws)    = Llambdaplustilde_temp(:,:,1:numDesireddraws-count);
        mv_save(:,count+1:numDesireddraws)                    = mv_temp(:,1:numDesireddraws-count);
        uw_save(:,count+1:numDesireddraws)                    = uw_temp(:,1:numDesireddraws-count);
        uw_uniform_save(:,count+1:numDesireddraws)            = uw_uniform_temp(:,1:numDesireddraws-count);
        
        record = record+1;
        display(['Number of draws = ',num2str(record*number)])
        display(['Number of successful draws = ',num2str(numDesireddraws)])
        
        doit = 1;
        
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step 4 of Algorithm 2: compute the normalized weights and estimate the effective sample size of the importance sampler
w_IP  = uw_save(1,:)/sum(uw_save(1,:));
ne    = floor(1/sum(w_IP.^2));

if uniform_prior == 1
    w_uniform_IP  = uw_uniform_save(1,:)/sum(uw_uniform_save(1,:));
    ne_uniform    = floor(1/sum(w_uniform_IP.^2));
    figure
    plot(w_IP)
    hold on
    plot(w_uniform_IP)
    w_IP = w_uniform_IP;
end

figure
subplot(2,2,1)
plot(uw_uniform_save(1,:)')
subplot(2,2,2)
plot(uw_uniform_save(4,:)')
subplot(2,2,3)
plot(uw_uniform_save(2,:)')
hold on
plot(uw_uniform_save(3,:)')

%% useful definitions to store relevant objects
A0tilde       = zeros(ntilde,ntilde,numDesireddraws);            % define array to store weighted \tilde{A}_0
Aplustilde    = zeros(ntilde*n_lags+nex,ntilde,numDesireddraws); % define array to store weighted \tilde{A}_plus

% initialize counter to track the state of the importance sampler
for s = 1:numDesireddraws
  
    is_draw             = randsample(1:size(w_IP,2),1,true,w_IP);
    A0tilde(:,:,s)      = A0tilde_save(:,:,is_draw);
    Aplustilde(:,:,s)   = Aplustilde_save(:,:,is_draw);   
      
end

%% IRF and FEVD
Draws_IRFs   = nan(ntilde,ntilde,Hz,numDesireddraws);
Draws_FEVDs  = nan(ntilde,ntilde,Hz,numDesireddraws);
Draws_Shocks = nan(T,ntilde,numDesireddraws);

for i = 1:numDesireddraws
    
    CT = [inv(A0tilde(:,:,i))'*Aplustilde(1:end-1,:,i)';eye((n_lags-1)*ntilde,n_lags*ntilde)];
    AT = [inv(A0tilde(:,:,i))';zeros(ntilde*n_lags-ntilde,ntilde)];   
    
    IRFs = nan(ntilde,ntilde,Hz);
    for h = 1:Hz
        irf = (CT^(h-1))*AT;
        IRFs(:,:,h) = irf(1:ntilde,:);
    end
    
    Draws_IRFs(:,:,:,i)  = IRFs;
    Draws_FEVDs(:,:,:,i) = varianceDecompositionOfVAR(IRFs,Hz-1);
    
    Draws_Shocks(:,:,i)  = Y*A0tilde_save(:,:,i) - X*Aplustilde_save(:,:,i);
           
end

%%
targetshock = [n-1;n];
LineWidthp  = 3;
bands68 = [16,84];
bands90 = [5,95];
plotx   = 0:1:Hz-1;
for j = 1:length(targetshock)
    targetshockj = targetshock(j);
    figure
    for i = 1:n
        subplot(ceil(n/3),3,i)
        relevantIRFs = squeeze(Draws_IRFs(i,targetshockj,:,:));
        medianIRF    = prctile(relevantIRFs,50,2);
        IRFband68    = prctile(relevantIRFs,bands68,2);
        IRFband90    = prctile(relevantIRFs,bands90,2);
        plot(plotx,medianIRF,'LineWidth',LineWidthp,'Color','b')
        hold on
        [~,~]=jbfill(plotx,IRFband68(:,1)',IRFband68(:,2)',rgb('mediumblue'),rgb('mediumblue'),0,0.3);
        hold on
        [~,~]=jbfill(plotx,IRFband90(:,1)',IRFband90(:,2)',rgb('mediumblue'),rgb('mediumblue'),0,0.1);
        hold on
        plot(plotx,zeros(Hz,1),'LineWidth',1,'Color',[0.5 0.5 0.5]);
        axis tight;
        title(varnames{i})
    end
end

for j = 1:length(targetshock)
    targetshockj = targetshock(j);
    figure
    for i = 1:n
        subplot(ceil(n/3),3,i)
        relevantFEVDs = squeeze(Draws_FEVDs(i,targetshockj,:,:));
        medianFEVD    = prctile(relevantFEVDs,50,2);
        FEVDband68    = prctile(relevantFEVDs,bands68,2);
        FEVDband90    = prctile(relevantFEVDs,bands90,2);
        plot(plotx,medianFEVD,'LineWidth',LineWidthp,'Color','b')
        hold on
        [~,~]=jbfill(plotx,FEVDband68(:,1)',FEVDband68(:,2)',rgb('mediumblue'),rgb('mediumblue'),0,0.3);
        hold on
        [~,~]=jbfill(plotx,FEVDband90(:,1)',FEVDband90(:,2)',rgb('mediumblue'),rgb('mediumblue'),0,0.1);
        hold on
        plot(plotx,zeros(Hz,1),'LineWidth',1,'Color',[0.5 0.5 0.5]);
        axis tight;
        title(varnames{i})
    end
end

datesp = dates(n_lags+1:end,:);
for j = 1:length(targetshock)
    figure
    i = targetshock(j);
    relevantshocks = squeeze(Draws_Shocks(:,i,:));
    medianshock    = prctile(relevantshocks,50,2);
    shockband68    = prctile(relevantshocks,bands68,2);
    shockband90    = prctile(relevantshocks,bands90,2);
    plot(datesp,medianshock,'LineWidth',LineWidthp,'Color','k')
    hold on
    jbfill(datesp',shockband68(:,1)',shockband68(:,2)',[0.5 0.5 0.5])
    hold on
    jbfill(datesp',shockband90(:,1)',shockband90(:,2)',[0.8 0.8 0.8])
    hold on
    plot(datesp,zeros(T,1),'LineWidth',1,'Color',[0.5 0.5 0.5]);
    datetick
    axis tight;
end

%% Save Result
save result_N A0tilde Aplustilde

%%
tend = datetime('now');    % end time
computingtime = tend-tstart; % computing time
disp(computingtime)